// OMAC - Open Music App Collaboration
// Free and unencumbered software - NO WARRANTY
// OMACApplicationRegistry.m
// Author: Nic Grant - apps@audeonic.com
// Revision: $Revision: 1.6 $, $Date: 2014/02/05 12:39:21 $
#import "OMACApplicationRegistry.h"
#if TARGET_OS_IPHONE

// I'm an old C programmer
#include <sys/stat.h>
#include <netdb.h>
#include <arpa/inet.h>

// magic internal constants - change these at your peril
static const char* versioning_hostname = "omac.z.softaudio.com";
static NSString* plist_download_url = @"http://dl.dropbox.com/u/3540898/omac_apps.plist";
static NSString* plist_name = @"omac_apps";
static NSString* plist_type = @"plist";

static OMACApplicationRegistry* instance = nil;
static NSString* app_name = nil;

// basic trace support
BOOL OMACTrace = NO;

// safe and auditable release of an object
#define HAPPY_RELEASE(obj) if (obj != nil) \
{ OMAC_TRACE(@"OMACRel - release %s", #obj); [obj release]; obj = nil; }

@implementation OMACApplicationRegistry
@synthesize delegate;
@synthesize version;
@synthesize author;
@synthesize dateUpdated;
@synthesize futureURL;

// INTERNAL - (re)load a plist file into the class
- (void)load
{
	OMAC_TRACE(@"OMACReg - (re)load plist file into object");
	
	// release dictionary and set to nil if already exists
	// ie. we'll reload it
	if (dic != nil)
	{
		OMAC_TRACE(@"OMACReg - release dictionary for reload");
		HAPPY_RELEASE(dic);
	}
	
	// load plist, first from cache and then from bundle
	struct stat st;
	BOOL file_exists = (stat([cache_pname UTF8String], &st) == 0);
	OMAC_TRACE(@"OMACReg - cache_pname: %@, exists: %i", cache_pname, file_exists);
	if (file_exists == YES)
		dic = [[NSDictionary alloc] initWithContentsOfFile:cache_pname];
	else
	{
		file_exists = (stat([bundle_pname UTF8String], &st) == 0);
		OMAC_TRACE(@"OMACReg - bundle_pname: %@, exists: %i", bundle_pname, file_exists);
		if (file_exists == YES)
			dic = [[NSDictionary alloc] initWithContentsOfFile:bundle_pname];
	}
	
	// if no plist files exist, flag in trace and stop loading
	if (dic == nil)
	{
		OMAC_TRACE(@"OMACReg - No data found, dictionary is nil");
		return;
	}
	
	// sanity check to ensure plist data is valid - dictionary at top
	// level will have at least 5 items
	OMAC_TRACE(@"OMACReg - Dictionary items: %lu", (unsigned long)dic.count);
	if (dic.count < 5)
	{
		OMAC_TRACE(@"OMACReg - less than 5 items in dictionary. Ignore");
		HAPPY_RELEASE(dic);
		return;
	}
	
	// what version did we load?
	version = [dic objectForKey:@"Version"];
	plist_version = [version intValue];
	
	// trace date and author
	dateUpdated = [dic objectForKey:@"DateUpdated"];
	author = [dic objectForKey:@"Author"];
	futureURL = [dic objectForKey:@"FutureURL"];
	
	// dump fields & full dic description to trace
	OMAC_TRACE(@"OMACReg - Loaded plist version '%@' (int: %u)", version, (unsigned int)plist_version);
	OMAC_TRACE(@"OMACReg - DateUpdated: %@", dateUpdated);
	OMAC_TRACE(@"OMACReg - Author: %@", author);
	OMAC_TRACE(@"OMACReg - FutureURL: %@", futureURL);
	OMAC_TRACE(@"OMACReg - Dictionary: %@", dic.description);
}

// INTERNAL - memory cleanup/object resetting
- (void)release_all_apps
{
	if (all_apps != nil)
	{
		// release the OMACApplication objects
		for (OMACApplication* app in all_apps)
			[app release];
	}
	HAPPY_RELEASE(all_apps);
	HAPPY_RELEASE(filtered_apps);
}

// INTERNAL - shim function to call delegate from main thread
- (void)delegate_interim
{
	[delegate OMACUpdateComplete:update_result];
}

// INTERNAL
// called either in background thread or main thread depending whether caller
// set a delegate value (or not)
- (enum eOMACUpdateStatus)update_selector
{
	// need an autorelease pool if we're in our background thread
	NSAutoreleasePool* pool = nil;
	if (delegate != nil)	pool = [[NSAutoreleasePool alloc] init];
	
	OMAC_TRACE(@"OMACReg - update_internal bg_thread: %i", (delegate == nil) ? 0 : 1);
	update_result = eOMACNoChange;
	
	// get version number from dns, or return with dns failure
	struct hostent* hp = gethostbyname(versioning_hostname);
	if (hp == 0) 
	{
		update_result = eOMACVersionCheckFailed;
		goto finish_up;
	}
	
	// convert dns address into a version number (from network byte order)
	UInt32 latest_version;	memcpy(&latest_version, hp->h_addr, sizeof(UInt32));
	latest_version = ntohl(latest_version);
	OMAC_TRACE(@"OMACReg - latest plist version: %u (0x%08x), our version: %u",
				  (unsigned int)latest_version, (unsigned int)latest_version, (unsigned int)plist_version);
	
	// check latest against our own, update if we're behind
	if (latest_version > plist_version)
	{
		// download the newest plist file from url which could be specified in
		// the plist itself in the FutureURL field
		NSString* url_str = plist_download_url;
		if (dic != nil)
		{
			NSString* future_url = [dic objectForKey:@"FutureURL"];
			if ([future_url length] > 10)
			{
				OMAC_TRACE(@"OMACReg - future url found in existing plist, use it");
				url_str = future_url;
			}
		}
		OMAC_TRACE(@"OMACReg - Updated plist available, url: %@", url_str);

		NSURL* url = [NSURL URLWithString:url_str];
		NSDictionary* new_dic = [NSDictionary dictionaryWithContentsOfURL:url];
		
		// sanity check the dictionary data in case it is corrupt
		if (new_dic == nil || new_dic.count < 5)
		{
			OMAC_TRACE(@"OMACReg - Failed to fetch update");
			update_result = eOMACFetchFailed;
			goto finish_up;
		}
		OMAC_TRACE(@"OMACReg - update retrieved OK");
		
		// save and write file (shouldn't expect write to fail but it could)
		if ([new_dic writeToFile:cache_pname atomically:YES] == NO)
		{
			OMAC_TRACE(@"OMACReg - Failed to write update");
			update_result = eOMACFetchFailed;
			goto finish_up;
		}
		
		// force recreate all & filter apps lists if they are not already nil
		[self release_all_apps];
		
		// an update was received successfully, so reload and return OK
		[self load];
		update_result = eOMACUpdated;
		goto finish_up;
	}

finish_up:
	OMAC_TRACE(@"OMACReg - update, completion, result: %i", update_result);
	
	// pre-populate filtered apps in bg thread
	[self filteredApps];
	
	// return to delegate on main thread
	if (delegate != nil)
	{
		OMAC_TRACE(@"OMACReg - returning to main thread");
		[self performSelectorOnMainThread:@selector(delegate_interim) withObject:nil waitUntilDone:NO];
	}

	// release pool if we created one
	if (pool != nil) [pool release];
	
	return update_result;
}

- (void)initialise
{
	OMAC_TRACE(@"OMACReg - initialise");
	
	// init members (ok, so they'll already be nil, but who knows)
	dic = nil;
	filtered_apps = nil;
	all_apps = nil;
	delegate = nil;

	// configure cache pathname
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
	NSString *cacheDirectory = [paths objectAtIndex:0];
	cache_pname = [[NSString alloc] initWithFormat:@"%@/%@.%@", 
									cacheDirectory, plist_name, plist_type];
	
	// configure bundle pathname
	NSString *filePath = [[NSBundle mainBundle] pathForResource:plist_name ofType:plist_type];
	bundle_pname = [[NSString alloc] initWithString:filePath];
}

+ (id)instanceWithAppName:(NSString*)name
{
	OMAC_TRACE(@"OMACReg - instanceWithAppName '%@'", name);
	
	// set app_name if name is passed and still nil
	if (app_name == nil)
	{
		if (name == nil || name.length == 0)
		{
			OMAC_TRACE(@"OMACReg - name must be set, returning nil");
			return nil;
		}
		
		app_name = [NSString stringWithString:name];
		OMAC_TRACE(@"OMACReg - app_name set to %@", app_name);
	}
	
	// alloc or return shared instance
	if (instance == nil)
	{
		instance = [OMACApplicationRegistry alloc];
		if (instance == nil)
		{
			OMAC_TRACE(@"OMACReg - could not alloc space for instance");
			return nil;
		}
		[instance initialise];
	}
	else
		return instance;

	// load in the current plist
	[instance load];
	
	return instance;
}

+ (void)shutdown
{
	OMAC_TRACE(@"OMACReg - shutdown");
	
	HAPPY_RELEASE(instance);
	HAPPY_RELEASE(app_name);
}
// more or less a shim function to either update on a background thread
// or synchronously depending upon whether the caller spacfified a delegate
- (enum eOMACUpdateStatus)update
{
	OMAC_TRACE(@"OMACReg - update, delegate: %@", delegate);
	
	// if delegate set, start update in new thread
	// else perform update on current thread and caller will wait
	if (delegate != nil)
	{
		[self performSelectorInBackground:@selector(update_selector) withObject:nil];
		return eOMACUpdateStarted;
	}

	return [self update_selector];
}

// create sorted (installed DESC, name ASC) array of all apps in plist
- (NSArray*)allApps
{
	OMAC_TRACE(@"OMACReg - allApps");
	
	// safety - if dictionary is nil, return nil array
	if (dic == nil) return nil;
	
	// create list only if needed
	if (all_apps == nil)
	{
		NSArray* dic_array = [dic objectForKey:@"Applications"];
		all_apps = [[NSMutableArray alloc] initWithCapacity:dic_array.count];
		OMAC_TRACE(@"OMACReg - plist has %lu apps", (unsigned long)dic_array.count);
		
		// cycle through the plist array and create OMACApplication objects
		// for each dictionary element
		for (NSDictionary* d in dic_array) 
		{
			OMACApplication* app = [[OMACApplication alloc] initWithDictionary:d];
			[all_apps addObject:app];
		}
		
		// sort the array by installed descending, name ascending
		[all_apps sortUsingSelector:@selector(compare:)];
	}
	
	return all_apps;
}

// subset array of apps in the plist
// excluded: our own app, unsupported devices and unsupported iOS versions
- (NSArray*)filteredApps
{
	OMAC_TRACE(@"OMACReg - filteredApps");

	// safety - if dictionary is nil, return nil array
	if (dic == nil) return nil;

	// create filtered list only if needed
	if (filtered_apps == nil)
	{
		NSString* model = [[UIDevice currentDevice] model];

		// get all apps (if needed), then create new subset array
		[self allApps];
		filtered_apps = [[NSMutableArray alloc] initWithCapacity:all_apps.count];
		
		for (OMACApplication* a in all_apps)
		{
			// exclude self
			if ([a.name caseInsensitiveCompare:app_name] == NSOrderedSame)
			{
				OMAC_TRACE(@"OMACReg - exclude %@ from filter list, self", a.name);
				continue;
			}
			
			// exclude unsupported device models
			BOOL model_supported = NO;
			for (NSString* m in a.supportedPlatforms)
			{
				if ([model compare:m] == NSOrderedSame)
				{
					model_supported = YES;
					break;
				}
			}
			if (model_supported == NO)
			{
				OMAC_TRACE(@"OMACReg - exclude %@ from filter list, unsupported model '%@'",
							  a.name, model);
				continue;
			}
			
			// exclude unsupported iOS versions
			if (kCFCoreFoundationVersionNumber < a.minimumCoreFoundation)
			{
				OMAC_TRACE(@"OMACReg - exclude %@ from filter list, unsupported iOS %f",
							  a.name, kCFCoreFoundationVersionNumber);
				continue;
			}
			
			// add app to filter - it qualifies
			[filtered_apps addObject:a];
		}
	}
	
	return filtered_apps;
}

- (NSArray*)filteredVirtualMIDIApps
{
	OMAC_TRACE(@"OMACReg - filteredVirtualMIDIApps");
	
	// safety - if dictionary is nil, return nil array
	if (dic == nil) return nil;
	
	// create filtered list only if needed
	if (filtered_vm_apps == nil)
	{
		// get all apps (if needed), then create new subset array
		[self filteredApps];
		filtered_vm_apps = [[NSMutableArray alloc] initWithCapacity:all_apps.count];
		
		for (OMACApplication* a in filtered_apps)
		{
			// exclude if no virtual MIDI
			if (a.virtualPortName == nil || [a.virtualPortName length] == 0)
			{
				OMAC_TRACE(@"OMACReg - exclude %@ from vm_filter list", a.name);
				continue;
			}
			
			// add app to filter - it qualifies
			[filtered_vm_apps addObject:a];
		}
	}
	
	return filtered_vm_apps;
}

- (NSArray*)appPorts
{
    return [dic objectForKey:@"ApplicationsPorts"];
}

- (void)processMIDIMessage:(unsigned char*)bytes :(unsigned short)length
{
	OMAC_TRACE(@"OMACReg - processMIDIMessage");

    // check this is a valid OMAC message
    if (bytes[0] == 0xF0 && bytes[1] == 0x1F && bytes[length-1] == 0xF7)
    {
        switch (bytes[2])
        {
            // app switch message
            case 0x00 :
                // verify message length is 6 and we're not in the background
                // since switch messages only work in foreground
                OMAC_TRACE(@"  possible app switch message. validate");
                if (length == 6 && [UIApplication sharedApplication].applicationState != UIApplicationStateBackground)
                {
                    // extract MSB and LSB and get correct registry entry
                    unsigned short serial = (bytes[3] << 7) + bytes[4];
                    OMAC_TRACE(@"  valid app switch message, serial: %i (%04X)", serial, serial);
                    for (OMACApplication* a in filtered_apps)
                        if (a.serialNo == serial)
                        {
                            OMAC_TRACE(@"  url for serial matched: %@ (%@)", a.launchURL, a.name);
                            if ([app_name compare:a.name] != NSOrderedSame)
                                [a launch];
                            return;
                        }
                    OMAC_TRACE(@"  no match found for serial");
                }
                break;
            
            default :
                OMAC_TRACE(@"OMACReg - unknown OMAC MIDI message received, 0x%02X", bytes[2]);
        }
    }
    else
    {
        OMAC_TRACE(@"OMACReg - badly formed MIDI message. Ignored");
    }
}

// reset the installed flag of our current app list
- (void)refresh
{
	OMAC_TRACE(@"OMACReg - refresh");
	for (OMACApplication* a in all_apps)
		[a check_installed];
}

- (void)dealloc 
{
	OMAC_TRACE(@"OMACReg - dealloc");

	HAPPY_RELEASE(bundle_pname);
	HAPPY_RELEASE(cache_pname);
	HAPPY_RELEASE(dic);
	
	[super dealloc];
}

@end

@implementation OMACApplication

@synthesize installed;
@synthesize name;
@synthesize developer;
@synthesize description;
@synthesize launchURL;
@synthesize virtualPortName;
@synthesize supportedPlatforms;
@synthesize minimumCoreFoundation;
@synthesize appStoreURL;
@synthesize videoURL;
@synthesize serialNo;
@synthesize supportsMIDISwitch;

- (id)initWithDictionary:(NSDictionary*)dic
{
	// populate the object with the dictionary
	name = [dic objectForKey:@"Application"];
	developer = [dic objectForKey:@"Developer"];
	description = [dic objectForKey:@"Description"];
	virtualPortName = [dic objectForKey:@"VirtualPortName"];
	launchURL = [dic objectForKey:@"LaunchURL"];
	minimumCoreFoundation = [[dic objectForKey:@"MinimumIOSBuild"] floatValue];
	supportedPlatforms = [dic objectForKey:@"Platforms"];
	appStoreURL = [dic objectForKey:@"AppStoreURL"];
	videoURL = [dic objectForKey:@"VideoURL"];
	serialNo = [[dic objectForKey:@"Serial"] intValue];
	supportsMIDISwitch = [[dic objectForKey:@"SupportsMIDISwitch"] boolValue];
	
	OMAC_TRACE(@"OMACApp - initWithDictionary - %@", name);
	
	[self check_installed];
	return self;
}

// switch to or launch an installed/running app
- (void)launch
{
	OMAC_TRACE(@"OMACApp - launch %@ - %@", name, launchURL);

	if (installed)
	{
		BOOL result = [[UIApplication sharedApplication] openURL:launch_url];
		OMAC_TRACE(@"OMACApp - launch %@, result: %i", name, result);
	}
}

- (void)check_installed
{
	// is the app installed?
	HAPPY_RELEASE(launch_url);
	launch_url = [[NSURL alloc] initWithString:launchURL];
	installed = [[UIApplication sharedApplication] canOpenURL:launch_url];
	OMAC_TRACE(@"OMACApp - check_installed %@, installed: %i", name, installed);
}

- (void)dealloc 
{
	OMAC_TRACE(@"OMACApp - dealloc %@", name);
	
	HAPPY_RELEASE(launch_url);
	[super dealloc];
}

// sort comparison selector for applications
- (NSComparisonResult)compare:(OMACApplication*)other 
{
	if (self.installed == other.installed)
		return [self.name compare:other.name];
	return (self.installed > other.installed ? NSOrderedAscending : NSOrderedDescending);
}


@end

#endif
/*
 
 $Log: OMACApplicationRegistry.m,v $
 Revision 1.6  2014/02/05 12:39:21  nic
 Update for MacOS X awareness (compiles but does nothing)

 Revision 1.4  2012/10/11 11:22:00  nic
 Added support for MIDI message switching

 Revision 1.3  2011/11/23 09:29:44  nic
 version 3, with NLog changes

 Revision 1.2  2011/11/10 11:50:40  nic
 Fixed bug in luanch method

 Revision 1.1.1.1  2011/11/09 12:54:48  nic
 Initial release

 
 */
